@@ -1,5 +1,5 @@
 /*
- * Copyright © 2007-2008 Gerd Kohlberger <lowfi@chello.at>
+ * Copyright © 2007-2009 Gerd Kohlberger <lowfi@chello.at>
  *
  * This file is part of Mousetweaks.
  *
@@ -49,8 +49,23 @@ enum {
     DOUBLE_CLICK
 };
 
+typedef struct _MtCliArgs {
+    gdouble  delay_time;
+    gdouble  dwell_time;
+    gchar   *mode;
+    gint     pos_x;
+    gint     pos_y;
+    gint     threshold;
+    gboolean delay_click;
+    gboolean dwell_click;
+    gboolean shutdown;
+    gboolean daemonize;
+    gboolean ctw;
+    gboolean no_animation;
+} MtCliArgs;
+
 static GdkScreen *
-mt_main_current_screen (MTClosure *mt)
+mt_main_current_screen (MtData *mt)
 {
     GdkScreen *screen;
 
@@ -73,7 +88,7 @@ mt_main_generate_motion_event (GdkScreen *screen, gint x, gint y)
 }
 
 static guint
-mt_main_check_mouse_orientation (MTClosure *mt, guint button)
+mt_main_check_mouse_orientation (MtData *mt, guint button)
 {
     if (mt->left_handed) {
 	if (button == 1)
@@ -85,10 +100,10 @@ mt_main_check_mouse_orientation (MTClosure *mt, guint button)
 }
 
 static void
-mt_main_generate_button_event (MTClosure *mt,
-			       guint      button,
-			       gint       type,
-			       gulong     delay)
+mt_main_generate_button_event (MtData *mt,
+			       guint   button,
+			       gint    type,
+			       gulong  delay)
 {
     button = mt_main_check_mouse_orientation (mt, button);
 
@@ -127,7 +142,7 @@ mt_main_generate_button_event (MTClosure *mt,
 }
 
 static void
-mt_main_set_cursor (MTClosure *mt, GdkCursorType type)
+mt_main_set_cursor (MtData *mt, GdkCursorType type)
 {
     GdkScreen *screen;
     GdkCursor *cursor;
@@ -142,7 +157,7 @@ mt_main_set_cursor (MTClosure *mt, GdkCursorType type)
 }
 
 static void
-dwell_restore_single_click (MTClosure *mt)
+dwell_restore_single_click (MtData *mt)
 {
     if (mt->dwell_mode == DWELL_MODE_CTW)
 	mt_ctw_set_clicktype (mt, DWELL_CLICK_TYPE_SINGLE);
@@ -151,7 +166,7 @@ dwell_restore_single_click (MTClosure *mt)
 }
 
 static void
-mt_main_do_dwell_click (MTClosure *mt)
+mt_main_do_dwell_click (MtData *mt)
 {
     guint clicktype;
 
@@ -193,7 +208,7 @@ mt_main_do_dwell_click (MTClosure *mt)
 }
 
 static inline gboolean
-below_threshold (MTClosure *mt, gint x, gint y)
+below_threshold (MtData *mt, gint x, gint y)
 {
     gint dx, dy;
 
@@ -204,7 +219,7 @@ below_threshold (MTClosure *mt, gint x, gint y)
 }
 
 static gboolean
-mt_main_analyze_gesture (MTClosure *mt)
+mt_main_analyze_gesture (MtData *mt)
 {
     gint x, y, gd, i, dx, dy;
 
@@ -253,7 +268,7 @@ mt_main_analyze_gesture (MTClosure *mt)
 }
 
 static void
-mt_main_draw_line (MTClosure *mt, gint x1, gint y1, gint x2, gint y2)
+mt_main_draw_line (MtData *mt, gint x1, gint y1, gint x2, gint y2)
 {
     GdkWindow *root;
     GdkGC *gc;
@@ -272,7 +287,7 @@ mt_main_draw_line (MTClosure *mt, gint x1, gint y1, gint x2, gint y2)
 }
 
 static void
-dwell_start_gesture (MTClosure *mt)
+dwell_start_gesture (MtData *mt)
 {
     GdkCursor *cursor;
     GdkWindow *root;
@@ -295,7 +310,7 @@ dwell_start_gesture (MTClosure *mt)
 }
 
 static void
-dwell_stop_gesture (MTClosure *mt)
+dwell_stop_gesture (MtData *mt)
 {
     if (mt->override_cursor)
 	gdk_pointer_ungrab (gtk_get_current_event_time ());
@@ -317,7 +332,7 @@ dwell_stop_gesture (MTClosure *mt)
 static void
 dwell_timer_finished (MtTimer *timer, gpointer data)
 {
-    MTClosure *mt = data;
+    MtData *mt = data;
 
     mt_cursor_manager_restore_all (mt_cursor_manager_get_default ());
 
@@ -357,7 +372,7 @@ eval_func (Accessible *a, gpointer data)
 static gboolean
 push_func (Accessible *a, gpointer data)
 {
-    MTClosure *mt = data;
+    MtData *mt = data;
     AccessibleRole role;
 
     role = Accessible_getRole (a);
@@ -374,7 +389,7 @@ push_func (Accessible *a, gpointer data)
 }
 
 static gboolean
-mt_main_use_move_release (MTClosure *mt)
+mt_main_use_move_release (MtData *mt)
 {
     Accessible *point, *search;
 
@@ -395,7 +410,7 @@ mt_main_use_move_release (MTClosure *mt)
 static gboolean
 right_click_timeout (gpointer data)
 {
-    MTClosure *mt = data;
+    MtData *mt = data;
 
     mt_main_generate_button_event (mt, 3, CLICK, CurrentTime);
 
@@ -405,7 +420,7 @@ right_click_timeout (gpointer data)
 static void
 delay_timer_finished (MtTimer *timer, gpointer data)
 {
-    MTClosure *mt = data;
+    MtData *mt = data;
     GdkScreen *screen;
 
     mt_cursor_manager_restore_all (mt_cursor_manager_get_default ());
@@ -428,13 +443,32 @@ delay_timer_finished (MtTimer *timer, gpointer data)
     g_timeout_add (100, right_click_timeout, data);
 }
 
+static void
+mt_dwell_click_cancel (MtData *mt)
+{
+    if (mt->dwell_gesture_started) {
+	dwell_stop_gesture (mt);
+	return;
+    }
+
+    mt_timer_stop (mt->dwell_timer);
+    mt_cursor_manager_restore_all (mt_cursor_manager_get_default ());
+
+    if (mt->dwell_drag_started) {
+	mt_main_set_cursor (mt, GDK_LEFT_PTR);
+	mt->dwell_drag_started = FALSE;
+    }
+
+    dwell_restore_single_click (mt);
+}
+
 /* at-spi listener callbacks */
 static void
 global_motion_event (MtListener *listener,
 		     MtEvent    *event,
 		     gpointer    data)
 {
-    MTClosure *mt = data;
+    MtData *mt = data;
 
     if (mt_timer_is_running (mt->delay_timer)) {
 	if (!below_threshold (mt, event->x, event->y)) {
@@ -471,29 +505,34 @@ global_button_event (MtListener *listener,
 		     MtEvent    *event,
 		     gpointer    data)
 {
-    MTClosure *mt = data;
+    MtData *mt = data;
 
-    if (event->type == EV_BUTTON_PRESS) {
-	if (mt->delay_enabled) {
+    if (mt->delay_enabled && event->button == 1) {
+	if (event->type == EV_BUTTON_PRESS) {
 	    mt->pointer_x = event->x;
 	    mt->pointer_y = event->y;
 	    mt_timer_start (mt->delay_timer);
 	}
-	if (mt->dwell_gesture_started)
-	    dwell_stop_gesture (mt);
-    }
-    else {
-	if (mt->delay_enabled) {
+	else {
 	    mt_timer_stop (mt->delay_timer);
 	    mt_cursor_manager_restore_all (mt_cursor_manager_get_default ());
 	}
     }
+    /*
+     * cancel a dwell-click in progress if a physical button
+     * is pressed - useful for mixed use-cases and testing
+     */
+    if ((event->type == EV_BUTTON_PRESS && mt_timer_is_running (mt->dwell_timer)) ||
+        (event->type == EV_BUTTON_RELEASE && mt->dwell_drag_started)) {
+	mt_dwell_click_cancel (mt);
+    }
 }
 
 static void
-global_focus_event (MtListener *listener, gpointer data)
+global_focus_event (MtListener *listener,
+		    gpointer    data)
 {
-    MTClosure *mt = data;
+    MtData *mt = data;
     Accessible *accessible;
 
     if (mt->delay_enabled) {
@@ -506,12 +545,12 @@ global_focus_event (MtListener *listener, gpointer data)
 }
 
 static gboolean
-cursor_overlay_time (MTClosure *mt,
-		     guchar    *image,
-		     gint       width,
-		     gint       height,
-		     MtTimer   *timer,
-		     gdouble    time)
+cursor_overlay_time (MtData  *mt,
+		     guchar  *image,
+		     gint     width,
+		     gint     height,
+		     MtTimer *timer,
+		     gdouble  time)
 {
     GtkWidget *ctw;
     GdkColor c;
@@ -551,9 +590,9 @@ cursor_overlay_time (MTClosure *mt,
 }
 
 static void
-mt_main_update_cursor (MTClosure *mt,
-		       MtTimer   *timer,
-		       gdouble    time)
+mt_main_update_cursor (MtData  *mt,
+		       MtTimer *timer,
+		       gdouble  time)
 {
     guchar *image;
     gushort width, height;
@@ -585,7 +624,7 @@ mt_main_timer_tick (MtTimer *timer,
 		    gdouble  time,
 		    gpointer data)
 {
-    MTClosure *mt = data;
+    MtData *mt = data;
 
     if (mt->animate_cursor && mt->cursor)
 	mt_main_update_cursor (mt, timer, time);
@@ -595,7 +634,7 @@ static void
 cursor_cache_cleared (MtCursorManager *manager,
 		      gpointer         data)
 {
-    MTClosure *mt = data;
+    MtData *mt = data;
 
     mt->cursor = mt_cursor_manager_current_cursor (manager);
 }
@@ -605,7 +644,7 @@ cursor_changed (MtCursorManager *manager,
 		const gchar     *name,
 		gpointer         data)
 {
-    MTClosure *mt = data;
+    MtData *mt = data;
 
     if (!mt->dwell_gesture_started)
 	mt->override_cursor = !g_str_equal (name, "left_ptr");
@@ -625,7 +664,7 @@ gconf_value_changed (GConfClient *client,
 		     GConfValue  *value,
 		     gpointer     data)
 {
-    MTClosure *mt = data;
+    MtData *mt = data;
 
     if (g_str_equal (key, OPT_THRESHOLD) && value->type == GCONF_VALUE_INT)
 	mt->threshold = gconf_value_get_int (value);
@@ -678,7 +717,7 @@ gconf_value_changed (GConfClient *client,
 }
 
 static void
-get_gconf_options (MTClosure *mt)
+get_gconf_options (MtData *mt)
 {
     gdouble val;
 
@@ -709,7 +748,7 @@ get_gconf_options (MTClosure *mt)
 }
 
 static void
-mt_main_request_logout (MTClosure *mt)
+mt_main_request_logout (MtData *mt)
 {
     DBusGConnection *bus;
     DBusGProxy *proxy;
@@ -732,8 +771,8 @@ mt_main_request_logout (MTClosure *mt)
 }
 
 static gboolean
-accessibility_enabled (MTClosure *mt,
-		       gint       spi_status)
+accessibility_enabled (MtData *mt,
+		       gint    spi_status)
 {
     gint ret;
 
@@ -763,19 +802,19 @@ accessibility_enabled (MTClosure *mt,
     return TRUE;
 }
 
-static MTClosure *
-mt_closure_init (void)
+static MtData *
+mt_data_init (void)
 {
-    MTClosure *mt;
+    MtData *mt;
     gint ev_base, err_base, maj, min;
 
-    mt = g_slice_new0 (MTClosure);
+    mt = g_slice_new0 (MtData);
     mt->xtst_display = XOpenDisplay (NULL);
 
     if (!XTestQueryExtension (mt->xtst_display,
 			      &ev_base, &err_base, &maj, &min)) {
 	XCloseDisplay (mt->xtst_display);
-	g_slice_free (MTClosure, mt);
+	g_slice_free (MtData, mt);
 	g_critical ("No XTest extension found. Aborting..");
 	return NULL;
     }
@@ -812,7 +851,7 @@ mt_closure_init (void)
 }
 
 static void
-mt_closure_free (MTClosure *mt)
+mt_data_free (MtData *mt)
 {
     g_object_unref (mt->delay_timer);
     g_object_unref (mt->dwell_timer);
@@ -824,62 +863,174 @@ mt_closure_free (MTClosure *mt)
 	g_object_unref (mt->ui);
     }
 
-    g_slice_free (MTClosure, mt);
+    g_slice_free (MtData, mt);
+}
+
+static MtCliArgs
+mt_parse_options (int *argc, char ***argv)
+{
+    MtCliArgs ca;
+    GOptionContext *context;
+    GOptionEntry entries[] = {
+	{"enable-dwell", 0, 0, G_OPTION_ARG_NONE, &ca.dwell_click,
+	    N_("Enable dwell click"), 0},
+	{"enable-secondary", 0, 0, G_OPTION_ARG_NONE, &ca.delay_click,
+	    N_("Enable simulated secondary click"), 0},
+	{"dwell-time", 0, 0, G_OPTION_ARG_DOUBLE, &ca.dwell_time,
+	    N_("Time to wait before a dwell click"), "[0.2-3.0]"},
+	{"secondary-time", 0, 0, G_OPTION_ARG_DOUBLE, &ca.delay_time,
+	    N_("Time to wait before a simulated secondary click"), "[0.5-3.0]"},
+	{"dwell-mode", 'm', 0, G_OPTION_ARG_STRING, &ca.mode,
+	    N_("Set the active dwell mode"), "[window|gesture]"},
+	{"show-ctw", 'c', 0, G_OPTION_ARG_NONE, &ca.ctw,
+	    N_("Show a click-type window"), 0},
+	{"ctw-x", 'x', 0, G_OPTION_ARG_INT, &ca.pos_x,
+	    N_("Click-type window X position"), 0},
+	{"ctw-y", 'y', 0, G_OPTION_ARG_INT, &ca.pos_y,
+	    N_("Click-type window Y position"), 0},
+	{"threshold", 't', 0, G_OPTION_ARG_INT, &ca.threshold,
+	    N_("Ignore small pointer movements"), "[0-30]"},
+	{"shutdown", 's', 0, G_OPTION_ARG_NONE, &ca.shutdown,
+	    N_("Shut down mousetweaks"), 0},
+	{"disable-animation", 0, 0, G_OPTION_ARG_NONE, &ca.no_animation,
+	    N_("Disable cursor animations"), 0},
+	{"daemonize", 0, 0, G_OPTION_ARG_NONE, &ca.daemonize,
+	    N_("Start mousetweaks as a daemon"), 0},
+	{ NULL }
+    };
+    /* init cli arguments */
+    ca.delay_time   = -1.;
+    ca.dwell_time   = -1.;
+    ca.mode         = NULL;
+    ca.pos_x        = -1;
+    ca.pos_y        = -1;
+    ca.threshold    = -1;
+    ca.delay_click  = FALSE;
+    ca.dwell_click  = FALSE;
+    ca.shutdown     = FALSE;
+    ca.daemonize    = FALSE;
+    ca.ctw          = FALSE;
+    ca.no_animation = FALSE;
+    /* parse */
+    context = g_option_context_new (_("- GNOME mouse accessibility daemon"));
+    g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
+    g_option_context_parse (context, argc, argv, NULL);
+    g_option_context_free (context);
+
+    return ca;
+}
+
+static void
+mt_main (int argc, char **argv, MtCliArgs cli_args)
+{
+    MtData *mt;
+    MtCursorManager *manager;
+    MtListener *listener;
+    gint spi_status;
+    gint spi_leaks = 0;
+
+    if (mt_pidfile_create () < 0) {
+	g_warning ("Couldn't create PID file.");
+	return;
+    }
+
+    signal (SIGINT, signal_handler);
+    signal (SIGTERM, signal_handler);
+    signal (SIGQUIT, signal_handler);
+    signal (SIGHUP, signal_handler);
+
+    gtk_init (&argc, &argv);
+
+    mt = mt_data_init ();
+    if (!mt)
+	goto FINISH;
+
+    spi_status = SPI_init ();
+    if (!accessibility_enabled (mt, spi_status)) {
+	mt_data_free (mt);
+	goto FINISH;
+    }
+
+    /* load gconf settings */
+    get_gconf_options (mt);
+
+    /* override with CLI arguments */
+    if (cli_args.dwell_click)
+	mt->dwell_enabled = cli_args.dwell_click;
+    if (cli_args.delay_click)
+	mt->delay_enabled = cli_args.delay_click;
+    if (cli_args.dwell_time >= .1 && cli_args.dwell_time <= 3.)
+	mt_timer_set_target (mt->dwell_timer, cli_args.dwell_time);
+    if (cli_args.delay_time >= .1 && cli_args.delay_time <= 3.)
+	mt_timer_set_target (mt->delay_timer, cli_args.delay_time);
+    if (cli_args.threshold >= 0 && cli_args.threshold <= 30)
+	mt->threshold = cli_args.threshold;
+    if (cli_args.ctw)
+	mt->dwell_show_ctw = cli_args.ctw;
+    if (cli_args.no_animation)
+	mt->animate_cursor = !cli_args.no_animation;
+    if (cli_args.mode) {
+	if (g_str_equal (cli_args.mode, "gesture"))
+	    mt->dwell_mode = DWELL_MODE_GESTURE;
+	else if (g_str_equal (cli_args.mode, "window"))
+	    mt->dwell_mode = DWELL_MODE_CTW;
+
+	g_free (cli_args.mode);
+    }
+
+    /* init click-type windoe */
+    if (!mt_ctw_init (mt, cli_args.pos_x, cli_args.pos_y))
+	goto CLEANUP;
+
+    /* init cursor animation */
+    manager = mt_cursor_manager_get_default ();
+    mt->cursor = mt_cursor_manager_current_cursor (manager);
+    g_signal_connect (manager, "cursor_changed",
+                      G_CALLBACK (cursor_changed), mt);
+    g_signal_connect (manager, "cache_cleared",
+                      G_CALLBACK (cursor_cache_cleared), mt);
+
+    /* init at-spi signals */
+    listener = mt_listener_get_default ();
+    g_signal_connect (listener, "motion_event",
+                      G_CALLBACK (global_motion_event), mt);
+    g_signal_connect (listener, "button_event",
+                      G_CALLBACK (global_button_event), mt);
+    g_signal_connect (listener, "focus_changed",
+                      G_CALLBACK (global_focus_event), mt);
+
+    gtk_main ();
+
+    mt_cursor_manager_restore_all (manager);
+    g_object_unref (manager);
+    g_object_unref (listener);
+
+CLEANUP:
+    spi_leaks = SPI_exit ();
+    mt_data_free (mt);
+FINISH:
+    mt_pidfile_remove ();
+
+    if (spi_leaks)
+	g_warning ("AT-SPI reported %i leak%s.",
+		   spi_leaks, spi_leaks != 1 ? "s" : "");
 }
 
 int
 main (int argc, char **argv)
 {
-    pid_t    pid;
-    gboolean delay_click = FALSE;
-    gboolean dwell_click = FALSE;
-    gdouble  delay_time  = -1.;
-    gdouble  dwell_time  = -1.;
-    gboolean shutdown    = FALSE;
-    gboolean ctw         = FALSE;
-    gboolean animate     = FALSE;
-    gchar   *mode        = NULL;
-    gint     pos_x       = -1;
-    gint     pos_y       = -1;
-    gint     threshold   = -1;
-    GOptionContext *context;
-    GOptionEntry entries[] = {
-	{"enable-dwell", 0, 0, G_OPTION_ARG_NONE, &dwell_click,
-	    N_("Enable dwell click"), 0},
-	{"enable-secondary", 0, 0, G_OPTION_ARG_NONE, &delay_click,
-	    N_("Enable simulated secondary click"), 0},
-	{"dwell-time", 0, 0, G_OPTION_ARG_DOUBLE, &dwell_time,
-	    N_("Time to wait before a dwell click"), "[0.2-3.0]"},
-	{"secondary-time", 0, 0, G_OPTION_ARG_DOUBLE, &delay_time,
-	    N_("Time to wait before a simulated secondary click"), "[0.5-3.0]"},
-	{"dwell-mode", 'm', 0, G_OPTION_ARG_STRING, &mode,
-	    N_("Dwell mode to use"), "[window|gesture]"},
-	{"show-ctw", 'c', 0, G_OPTION_ARG_NONE, &ctw,
-	    N_("Show click type window"), 0},
-	{"ctw-x", 'x', 0, G_OPTION_ARG_INT, &pos_x,
-	    N_("Window x position"), 0},
-	{"ctw-y", 'y', 0, G_OPTION_ARG_INT, &pos_y,
-	    N_("Window y position"), 0},
-	{"threshold", 't', 0, G_OPTION_ARG_INT, &threshold,
-	    N_("Ignore small pointer movements"), "[0-30]"},
-	{"animate-cursor", 'a', 0, G_OPTION_ARG_NONE, &animate,
-	    N_("Show elapsed time as cursor overlay"), 0},
-	{"shutdown", 's', 0, G_OPTION_ARG_NONE, &shutdown,
-	    N_("Shut down mousetweaks"), 0},
-	{ NULL }
-    };
+    MtCliArgs cli_args;
+    pid_t pid;
 
     bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
     bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
     textdomain (GETTEXT_PACKAGE);
     setlocale (LC_ALL, "");
 
-    context = g_option_context_new (_("- GNOME mousetweaks daemon"));
-    g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
-    g_option_context_parse (context, &argc, &argv, NULL);
-    g_option_context_free (context);
+    g_set_application_name ("Mousetweaks");
+    cli_args = mt_parse_options (&argc, &argv);
 
-    if (shutdown) {
+    if (cli_args.shutdown) {
 	int ret;
 
 	if ((ret = mt_pidfile_kill_wait (SIGINT, 5)) < 0)
@@ -889,112 +1040,29 @@ main (int argc, char **argv)
 
 	return ret < 0 ? 1 : 0;
     }
-    if ((pid = mt_pidfile_is_running ()) >= 0) {
-	g_print ("Daemon is already running. (PID %u)\n", pid);
-	return 1;
-    }
-    g_print ("Starting daemon.\n");
 
-    if ((pid = fork ()) < 0) {
-	g_error ("fork() failed.");
+    if ((pid = mt_pidfile_is_running ()) >= 0) {
+	g_print ("Mousetweaks is already running. (PID %u)\n", pid);
 	return 1;
     }
-    else if (pid) {
-	/* Parent return */
-	return 0;
+
+    if (cli_args.daemonize) {
+	g_print ("Starting daemon.\n");
+	if ((pid = fork ()) < 0) {
+	    g_error ("fork() failed.");
+	    return 1;
+	}
+	else if (pid) {
+	    /* Parent return */
+	    return 0;
+	}
+	else {
+	    /* Child process */
+	    mt_main (argc, argv, cli_args);
+	}
     }
     else {
-	/* Child process */
-	MTClosure *mt;
-	MtCursorManager *manager;
-	MtListener *listener;
-	gint spi_status;
-	gint spi_leaks = 0;
-
-	if (mt_pidfile_create () < 0)
-	    return 1;
-
-	signal (SIGINT, signal_handler);
-	signal (SIGTERM, signal_handler);
-	signal (SIGQUIT, signal_handler);
-	signal (SIGHUP, signal_handler);
-
-	gtk_init (&argc, &argv);
-	g_set_application_name ("Mousetweaks");
-
-	mt = mt_closure_init ();
-	if (!mt)
-	    goto FINISH;
-
-	spi_status = SPI_init ();
-	if (!accessibility_enabled (mt, spi_status)) {
-	    mt_closure_free (mt);
-	    goto FINISH;
-	}
-
-	/* command-line options */
-	if (dwell_click)
-	    gconf_client_set_bool (mt->client, OPT_DWELL, TRUE, NULL);
-	if (delay_click)
-	    gconf_client_set_bool (mt->client, OPT_DELAY, TRUE, NULL);
-	if (delay_time >= .5 && delay_time <= 3.)
-	    gconf_client_set_float (mt->client, OPT_DELAY_T, delay_time, NULL);
-	if (dwell_time >= .2 && dwell_time <= 3.)
-	    gconf_client_set_float (mt->client, OPT_DWELL_T, dwell_time, NULL);
-	if (threshold >= 0 && threshold <= 30)
-	    gconf_client_set_int (mt->client, OPT_THRESHOLD, threshold, NULL);
-	if (ctw)
-	    gconf_client_set_bool (mt->client, OPT_CTW, TRUE, NULL);
-	if (animate)
-	    gconf_client_set_bool (mt->client, OPT_ANIMATE, TRUE, NULL);
-	if (mode) {
-	    if (g_str_equal (mode, "gesture"))
-		gconf_client_set_int (mt->client, OPT_MODE,
-				      DWELL_MODE_GESTURE, NULL);
-	    else if (g_str_equal (mode, "window"))
-		gconf_client_set_int (mt->client, OPT_MODE,
-				      DWELL_MODE_CTW, NULL);
-	    g_free (mode);
-	}
-
-	get_gconf_options (mt);
-
-	if (!mt_ctw_init (mt, pos_x, pos_y))
-	    goto CLEANUP;
-
-	/* init cursor animation */
-	manager = mt_cursor_manager_get_default ();
-	g_signal_connect (manager, "cursor_changed",
-			  G_CALLBACK (cursor_changed), mt);
-	g_signal_connect (manager, "cache_cleared",
-			  G_CALLBACK (cursor_cache_cleared), mt);
-
-	mt->cursor = mt_cursor_manager_current_cursor (manager);
-
-	/* init at-spi signals */
-	listener = mt_listener_get_default ();
-	g_signal_connect (listener, "motion_event",
-			  G_CALLBACK (global_motion_event), mt);
-	g_signal_connect (listener, "button_event",
-			  G_CALLBACK (global_button_event), mt);
-	g_signal_connect (listener, "focus_changed",
-			  G_CALLBACK (global_focus_event), mt);
-
-	gtk_main ();
-
-	mt_cursor_manager_restore_all (manager);
-	g_object_unref (manager);
-	g_object_unref (listener);
-
-	CLEANUP:
-	    spi_leaks = SPI_exit ();
-	    mt_closure_free (mt);
-	FINISH:
-	    mt_pidfile_remove ();
-
-	if (spi_leaks)
-	    g_warning ("AT-SPI reported %i leak%s.",
-		       spi_leaks, spi_leaks != 1 ? "s" : "");
+	mt_main (argc, argv, cli_args);
     }
     return 0;
 }
